
/* UNIX shadow file access module */

#include "Python.h"

#include <sys/types.h>
#include <shadow.h>

static char shadow__doc__ [] = "\
This module provides access to the Unix shadow password database.\n\
It is available on all Unix versions.\n\
\n\
Password database entries are reported as 7-tuples containing the following\n\
items from the password database (see `<shadow.h>'), in order:\n\
sp_namp, sp_pwdp, sp_lstchg, sp_min, sp_max, sp_warn, sp_inact, sp_expire, sp_flag.\n\
The namp and pwdp items are strings, the rest are integers. An\n\
exception is raised if the entry asked for cannot be found.";

      
static PyObject *
mkshadowent(struct spwd *s)
{
	return Py_BuildValue(
		"(sslllllll)",
		s->sp_namp,
		s->sp_pwdp,
    s->sp_lstchg,
    s->sp_min,
    s->sp_max,
    s->sp_warn,
    s->sp_inact,
    s->sp_expire,
    s->sp_flag);
}

static char shadow_getspnam__doc__[] = "\
getpwnam(name) -> entry\n\
Return the shadow database entry for the given user name.\n\
See shadow.__doc__ for more on password database entries.";

static PyObject *
shadow_getspnam(PyObject *self, PyObject *args)
{
	char *name;
	struct spwd *s;
	if (!PyArg_Parse(args, "s", &name))
		return NULL;
	if ((s = getspnam(name)) == NULL) {
		PyErr_SetString(PyExc_KeyError, "getspnam(): name not found");
		return NULL;
	}
	return mkshadowent(s);
}

static char shadow_getspall__doc__[] = "\
getpwall() -> list_of_entries\n\
Return a list of all available shadow database entries, \
in arbitrary order.\n\
See shadow.__doc__ for more on password database entries.";

static PyObject *
shadow_getspall(PyObject *self, PyObject *args)
{
	PyObject *d;
	struct spwd *s;
	if (!PyArg_NoArgs(args))
		return NULL;
	if ((d = PyList_New(0)) == NULL)
		return NULL;
	setspent();
	while ((s = getspent()) != NULL) {
		PyObject *v = mkshadowent(s);
		if (v == NULL || PyList_Append(d, v) != 0) {
			Py_XDECREF(v);
			Py_DECREF(d);
			return NULL;
		}
		Py_DECREF(v);
	}
	endspent();
	return d;
}

static PyMethodDef shadow_methods[] = {
	{"getspnam",	shadow_getspnam, METH_OLDARGS, shadow_getspnam__doc__},
	{"getspall",	shadow_getspall, METH_OLDARGS, shadow_getspall__doc__},
	{NULL,		NULL}		/* sentinel */
};

DL_EXPORT(void)
initshadow(void)
{
	Py_InitModule4("shadow", shadow_methods, shadow__doc__,
                       (PyObject *)NULL, PYTHON_API_VERSION);
}
